
#include "Widget.h"
#include "WidgetDriver.h"
#include <cassert>

namespace ghost{

	namespace{

		ObjectPool<Widget>& GetWidgetPool_()
		{
			static ObjectPool<Widget> pool;
			return pool;
		}

	} // namespace

	struct WidgetImpl{
		widget::Driver_* pDriver;

		WidgetImpl()
			: pDriver(0)
		{

		}
	};

	Widget::Widget(HWND hWnd) 
		: pImpl_(new WidgetImpl)
	{
		pImpl_->pDriver = new widget::Driver_(hWnd);
		pImpl_->pDriver->SetRootWidget(this);
	}

	Widget::Widget(Widget* pParent) 
		: pImpl_(new WidgetImpl)
	{
		pImpl_->pDriver = pParent->pImpl_->pDriver;
	}

	Widget::~Widget()
	{
		if (IsRoot())
		{
			delete pImpl_->pDriver;
		}
		delete pImpl_;
	}

	Widget* Widget::Create(HWND hWnd)
	{
		return GetWidgetPool_().Construct(hWnd);
	}

	Widget* Widget::Create(Widget* pParent)
	{
		assert(pParent);
		if (pParent)
		{
			return GetWidgetPool_().Construct(pParent);
		}
		return 0;
	}

	void Widget::Destroy()
	{
		GetWidgetPool_().Destroy(this);
	}

	HWND Widget::GetContainerWindow() const
	{
		return pImpl_->pDriver->GetContainerWindow();
	}

	Widget* Widget::GetRoot() const
	{
		return pImpl_->pDriver->GetRootWidget();
	}

	bool Widget::IsRoot() const
	{
		return pImpl_->pDriver->GetRootWidget() == this;
	}

} // namespace ghost